@@ -4,6 +4,7 @@ Nomos is an open-source, configurable multi-step agent framework for building ad
 Define your agent's persona, tools, and step-by-step flows in Python or YAML—perfect for conversational, workflow, and automation use cases.
 """
 
+from .client import AuthConfig, NomosClient, NomosClientSync
 from .config import AgentConfig, ServerConfig
 from .core import Agent
 from .models.agent import Action, Route, State, Step, StepIdentifier, Summary
@@ -13,7 +14,7 @@ from .state_machine import StateMachine
 from .testing import smart_assert
 from .testing.e2e import Scenario, ScenarioRunner
 
-__version__ = "0.3.2"
+__version__ = "0.3.6"
 __author__ = "DoWhile"
 
 __all__ = [
@@ -36,4 +37,8 @@ __all__ = [
     "ScenarioRunner",
     "Scenario",
     "StateMachine",
+    # Client classes
+    "NomosClient",
+    "NomosClientSync",
+    "AuthConfig",
 ]
