@@ -25,7 +25,7 @@ from nomos.llms.openai import OpenAI
 from .tools import tool_list
 
 config = nomos.AgentConfig.from_yaml(os.getenv("CONFIG_PATH", "config.agent.yaml"))
-llm = config.llm.get_llm() if hasattr(config, "llm") and config.llm else OpenAI()
+llm = config.get_llm() if hasattr(config, "llm") and config.llm else OpenAI()
 agent = nomos.Agent.from_config(config, llm, tool_list)
 
-__all__ = ["agent"]
+__all__ = ["agent", "config"]
