@@ -7,8 +7,10 @@ from sqlalchemy.ext.asyncio.engine import create_async_engine
 from sqlmodel import SQLModel
 from sqlmodel.ext.asyncio.session import AsyncSession
 
+from .agent import config
+
 # Database configuration
-DATABASE_URL = os.getenv("DATABASE_URL")
+DATABASE_URL = os.getenv("DATABASE_URL") or config.server.session.database_url
 engine = create_async_engine(DATABASE_URL) if DATABASE_URL else None
 
 
