@@ -6,12 +6,12 @@ import importlib
 import importlib.util
 import os
 from enum import Enum
-from typing import Callable, Dict, List, Optional, Union
+from typing import Callable, Dict, List, Literal, Optional, Union
 
 from pydantic import BaseModel, Field
 from pydantic_settings import BaseSettings
 
-from .llms import LLMBase, LLMConfig, OpenAI
+from .llms import LLMBase, LLMConfig
 from .memory import MemoryConfig
 from .models.agent import Step
 from .models.flow import FlowConfig
@@ -50,14 +50,42 @@ class SessionConfig(BaseModel):
         )
 
 
+class ServerSecurity(BaseModel):
+    """Security configuration for the FastAPI server."""
+
+    # CORS configuration
+    allowed_origins: List[str] = Field(["*"], description="List of allowed origins for CORS")
+
+    # Authentication configuration
+    enable_auth: bool = False  # Flag to enable authentication
+    auth_type: Optional[Literal["jwt", "api_key"]] = None  # Type of authentication to use
+    jwt_secret_key: Optional[str] = None  # Secret key for JWT authentication
+    api_key_url: Optional[str] = None  # URL for API key validation
+
+    # Rate limiting configuration
+    enable_rate_limiting: bool = False  # Flag to enable rate limiting
+    redis_url: Optional[str] = None  # Redis URL for rate limiting
+    rate_limit_times: Optional[int] = None  # Number of allowed requests per time period
+    rate_limit_seconds: Optional[int] = None  # Time period for rate limiting in seconds
+
+    # CSRF protection configuration
+    enable_csrf_protection: bool = False  # Flag to enable CSRF protection
+    csrf_secret_key: Optional[str] = None  # Secret key for CSRF protection
+
+    # Development/Testing configuration
+    enable_token_endpoint: bool = (
+        False  # Flag to enable JWT token generation endpoint (dev/test only)
+    )
+
+
 class ServerConfig(BaseModel):
     """Configuration for the FastAPI server."""
 
-    redis_url: Optional[str] = None
-    database_url: Optional[str] = None
-    enable_tracing: bool = False
     port: int = 8000
+    host: str = "0.0.0.0"
     workers: int = 1
+    security: ServerSecurity = ServerSecurity()
+    session: SessionConfig = SessionConfig()
 
 
 class ExternalTool(BaseModel):
@@ -70,6 +98,9 @@ class ExternalTool(BaseModel):
     kwargs: Optional[Dict[str, Union[str, int, float]]] = (
         None  # Optional keyword arguments for the Tool initialization
     )
+    map: Optional[Dict[str, str]] = (
+        None  # Optional mapping of method names to API endpoints (for API tools)
+    )
 
     def get_tool_wrapper(self) -> ToolWrapper:
         """
@@ -81,30 +112,46 @@ class ExternalTool(BaseModel):
         tool_type = tool_type.replace("@", "")
         if tool_type == "mcp" and not self.name:
             raise ValueError("For MCP tools, the 'name' field is required.")
-
-        name = self.name or convert_camelcase_to_snakecase(tool_name.split(".")[-1])
+        assert not (tool_type == "api" and not self.map and not self.name), (
+            "For Direct API tools, the 'name' field is required."
+        )
+        name = (
+            self.name
+            or ("Multi-Endpoint API Tool" if self.map else None)
+            or convert_camelcase_to_snakecase(tool_name.split(".")[-1])
+        )
         assert tool_type in [
             "pkg",
             "crewai",
             "langchain",
             "mcp",
+            "api",
         ], (
-            f"Unsupported tool type: {tool_type}. Supported types are 'pkg', 'crewai', 'mcp' and 'langchain'."
+            f"Unsupported tool type: {tool_type}. Supported types are 'pkg', 'crewai', 'mcp', 'langchain', and 'api'."
         )
         return ToolWrapper(
             name=name,
             tool_type=tool_type,
             tool_identifier=tool_name,
             kwargs=self.kwargs,
+            map=self.map,
         )
 
 
 class ToolsConfig(BaseModel):
     """Configuration for tools used by the agent."""
 
-    tool_files: List[str] = []  # List of tool files to load
-    external_tools: Optional[List[ExternalTool]] = None  # List of external tools
-    tool_defs: Optional[Dict[str, ToolDef]] = None
+    tool_files: List[str] = Field(
+        default_factory=list, validation_alias="files", serialization_alias="files"
+    )
+    external_tools: Optional[List[ExternalTool]] = Field(
+        None, validation_alias="ext", serialization_alias="ext"
+    )  # List of external tools
+    tool_defs: Optional[Dict[str, ToolDef]] = Field(
+        None, validation_alias="defs", serialization_alias="defs"
+    )
+
+    model_config = {"populate_by_name": True}
 
     def get_tools(self) -> List[Union[Callable, ToolWrapper]]:
         """
@@ -204,7 +251,7 @@ class AgentConfig(BaseSettings):
     max_examples: int = 5  # Maximum number of examples to use in decision-making
     threshold: float = 0.5  # Minimum similarity score to include an example
 
-    llm: Optional[LLMConfig] = None  # Optional LLM configuration
+    llm: Optional[LLMConfig | Dict[str, LLMConfig]] = None  # Optional LLM configuration
     embedding_model: Optional[LLMConfig] = None  # Optional embedding model configuration
     memory: Optional[MemoryConfig] = None  # Optional memory configuration
     flows: Optional[List[FlowConfig]] = None  # Optional flow configurations
@@ -212,9 +259,6 @@ class AgentConfig(BaseSettings):
     server: ServerConfig = ServerConfig()  # Configuration for the FastAPI server
     tools: ToolsConfig = ToolsConfig()  # Configuration for tools
 
-    # Optional session store configuration
-    session: Optional[SessionConfig] = None
-
     logging: Optional[LoggingConfig] = None  # Optional logging configuration
 
     @classmethod
@@ -227,23 +271,24 @@ class AgentConfig(BaseSettings):
         """
         import yaml
 
+        def expand_env_vars(obj):
+            """Recursively expand environment variables in nested dictionaries and lists."""
+            if isinstance(obj, dict):
+                return {k: expand_env_vars(v) for k, v in obj.items()}
+            elif isinstance(obj, list):
+                return [expand_env_vars(item) for item in obj]
+            elif isinstance(obj, str) and obj.startswith("$"):
+                # Extract environment variable name and get its value
+                env_var = obj[1:]  # Remove the $ prefix
+                return os.getenv(env_var, obj)  # Return original value if env var not found
+            else:
+                return obj
+
         with open(file_path, "r") as file:
             data = yaml.safe_load(file)
-        server_data = data.get("server", {})
-        if isinstance(server_data, dict):
-            expanded = {
-                k: (os.getenv(v[1:], v) if isinstance(v, str) and v.startswith("$") else v)
-                for k, v in server_data.items()
-            }
-            data["server"] = expanded
 
-        session_data = data.get("session") or data.get("session_store")
-        if isinstance(session_data, dict):
-            expanded = {
-                k: (os.getenv(v[1:], v) if isinstance(v, str) and v.startswith("$") else v)
-                for k, v in session_data.items()
-            }
-            data["session"] = expanded
+        # Expand environment variables in the entire configuration
+        data = expand_env_vars(data)
 
         return cls(**data)
 
@@ -258,13 +303,21 @@ class AgentConfig(BaseSettings):
         with open(file_path, "w") as file:
             yaml.dump(self.model_dump(mode="json"), file, sort_keys=False)
 
-    def get_llm(self) -> LLMBase:
+    def get_llm(self) -> Optional[Dict[str, LLMBase]]:
         """
         Get the appropriate LLM instance based on the configuration.
+        :param id: ID of the LLM to retrieve.
 
         :return: An instance of the defined LLM integration.
         """
-        return self.llm.get_llm() if self.llm else OpenAI()
+        if not self.llm:
+            return None
+        llm_dict = (
+            {llm_id: llm.get_llm() for llm_id, llm in self.llm.items()}
+            if isinstance(self.llm, dict)
+            else {"global": self.llm.get_llm()}
+        )  # type: ignore
+        return llm_dict
 
     def get_embedding_model(self) -> Optional[LLMBase]:
         """
