@@ -40,14 +40,14 @@ class Session:
     def __init__(
         self,
         name: str,
-        llm: LLMBase,
+        llm: Union[LLMBase, Dict[str, LLMBase]],
         embedding_model: LLMBase,
         memory: Memory,
         steps: Dict[str, Step],
         start_step_id: str,
+        tools: Dict[str, Tool],
         system_message: Optional[str] = None,
         persona: Optional[str] = None,
-        tools: Optional[List[Union[Callable, ToolWrapper]]] = None,
         flows: Optional[List[Flow]] = None,
         show_steps_desc: bool = False,
         max_errors: int = 3,
@@ -75,7 +75,7 @@ class Session:
         # Fixed
         self.session_id = state.session_id if state else f"{name}_{str(uuid.uuid4())}"
         self.name = name
-        self.llm = llm
+        self.llm_dict = llm if isinstance(llm, dict) else {"global": llm}
         self.steps = steps
         self.show_steps_desc = show_steps_desc
         self.system_message = system_message
@@ -94,11 +94,8 @@ class Session:
         )
         self.embedding_model = embedding_model
 
-        tool_defs = (
-            self.config.tools.tool_defs if self.config and self.config.tools.tool_defs else None
-        )
         self.deferred_tools: Dict[str, Tool] = {}
-        self.tools: Dict[str, Tool] = get_tools(tools, tool_defs)
+        self.tools: Dict[str, Tool] = tools
 
         # Compile state machine for fast transitions and flow lookups
         self.state_machine = StateMachine(
@@ -245,6 +242,17 @@ class Session:
         self.current_step.reduce_tools(deferred_tool_names)
         return tuple(tools)
 
+    @property
+    def llm(self) -> LLMBase:
+        """
+        Get the LLM instance.
+        :return: The LLM instance.
+        """
+        llm_id = self.current_step.llm or "global"
+        if llm_id not in self.llm_dict:
+            log_error(f"LLM '{llm_id}' not found in session LLMs. Using default LLM.")
+        return self.llm_dict.get(llm_id, self.llm_dict["global"])
+
     def _add_event(
         self, event_type: str, content: str, decision: Optional[Decision] = None
     ) -> None:
@@ -308,6 +316,7 @@ class Session:
             flow_memory = self.state_machine.current_flow.get_memory()
             if flow_memory and isinstance(flow_memory, FlowMemoryComponent):
                 flow_memory_context = flow_memory.memory.context
+
         _decision = self.llm._get_output(
             steps=self.steps,
             current_step=self.current_step,
@@ -592,7 +601,7 @@ class Agent:
 
     def __init__(
         self,
-        llm: LLMBase,
+        llm: Union[LLMBase, Dict[str, LLMBase]],
         name: str,
         steps: List[Step],
         start_step_id: str,
@@ -609,7 +618,7 @@ class Agent:
         """
         Initialize an Agent.
 
-        :param llm: LLMBase instance.
+        :param llm: LLMBase instance or dictionary of LLMs.
         :param name: Name of the agent.
         :param steps: List of Step objects.
         :param start_step_id: ID of the starting step.
@@ -634,8 +643,11 @@ class Agent:
         self.max_iter = max_iter
         self.config = config
         self.embedding_model = (
-            embedding_model or (config.get_embedding_model() if config else None) or self.llm
+            embedding_model
+            or (config.get_embedding_model() if config else None)
+            or (llm if isinstance(llm, LLMBase) else llm.get("global", None))
         )
+        assert self.embedding_model, "Embedding model must be provided or configured."
         self._setup_logging()
         self.flows = flows or (
             list(create_flows_from_config(config).flows.values())
@@ -645,7 +657,7 @@ class Agent:
 
         # Remove duplicates of tools based on their names or IDs
         seen = set()
-        self.tools = []
+        _tools = []
         for tool in tools or []:
             tool_id = (
                 tool.name if isinstance(tool, ToolWrapper) else getattr(tool, "__name__", None)
@@ -653,7 +665,12 @@ class Agent:
             tool_id = tool_id or id(tool)  # Fallback to id if no name
             if tool_id not in seen:
                 seen.add(tool_id)
-                self.tools.append(tool)
+                _tools.append(tool)
+
+        del seen  # Clear the seen set to free memory
+
+        self.tools = get_tools(_tools, config.tools.tool_defs if config and config.tools else None)
+        del _tools  # Clear the temporary list to free memory
 
         # Validate start step ID
         if start_step_id not in self.steps:
@@ -669,19 +686,14 @@ class Agent:
                     raise ValueError(
                         f"Route target {route.target} not found in steps for step {step.step_id}"
                     )
+
         # Validate tool names
         for step in self.steps.values():
-            for step_tool in step.available_tools:
-                for tool in self.tools:
-                    if (isinstance(tool, ToolWrapper) and tool.id == step_tool) or (
-                        callable(tool) and getattr(tool, "__name__", None) == step_tool
-                    ):
-                        break
-                else:
-                    log_error(f"Tool {step_tool} not found in tools for step {step.step_id}")
-                    raise ValueError(
-                        f"Tool {step_tool} not found in tools for step {step.step_id}\nAvailable tools: {self.tools}"
-                    )
+            # check if the step.available tools is subset of the session tools
+            if not set(step.available_tools).issubset(set(self.tools.keys())):
+                err_msg = f"Step {step.step_id} has tools that are not defined in agent tools"
+                log_error(err_msg)
+                raise ValueError(err_msg)
 
         # Go through all the steps and if there are examples in them, perform batch embedding
         for step in self.steps.values():
@@ -693,25 +705,24 @@ class Agent:
     def from_config(
         cls,
         config: AgentConfig,
-        llm: Optional[LLMBase] = None,
+        llm: Optional[Union[LLMBase, Dict[str, LLMBase]]] = None,
         tools: Optional[List[Union[Callable, ToolWrapper]]] = None,
     ) -> "Agent":
         """
         Create an Agent from an AgentConfig object.
 
-        :param llm: LLMBase instance.
+        :param llm: Optional LLMBase instance or dictionary of LLMs.
         :param config: AgentConfig instance.
         :param tools: List of tool callables.
         :return: Nomos instance.
         """
-        if not llm:
-            if not config.llm:
-                raise ValueError("No LLM provided. Please provide an LLM or a config with an LLM.")
-            llm = config.llm.get_llm()
+        _llm = llm or config.get_llm()
+        assert _llm, "LLM must be provided either as a parameter or in the config."
+
         tools = tools or []
         tools.extend(config.tools.get_tools())
         return cls(
-            llm=llm,
+            llm=_llm,
             name=config.name,
             steps=config.steps,
             start_step_id=config.start_step_id,
@@ -745,6 +756,7 @@ class Agent:
             memory = (
                 self.config.memory.get_memory() if self.config and self.config.memory else Memory()
             )
+        assert self.embedding_model, "Embedding model must be provided or configured."
         return Session(
             name=self.name,
             llm=self.llm,
@@ -783,6 +795,7 @@ class Agent:
 
         memory = self.config.memory.get_memory() if self.config and self.config.memory else Memory()
 
+        assert self.embedding_model, "Embedding model must be provided or configured."
         session = Session(
             name=self.name,
             llm=self.llm,
@@ -865,7 +878,7 @@ class Agent:
             raise ValueError("save_path must be provided if not in a notebook environment.")
 
         mm_code = create_mermaid_graph(
-            steps=self.steps, flows=self.flows if self.flows else [], tools=get_tools(self.tools)
+            steps=self.steps, flows=self.flows if self.flows else [], tools=self.tools
         )
         mermaid_svg(
             mm_code,
