@@ -2,12 +2,14 @@
 
 import os
 from pathlib import Path
-from unittest.mock import MagicMock, patch
+from unittest.mock import MagicMock, Mock, patch
 
 import pytest
 
 from nomos.config import AgentConfig, ToolsConfig
 from nomos.core import Agent, Session
+from nomos.llms import LLMConfig
+from nomos.llms.base import LLMBase
 from nomos.models.agent import (
     Action,
     Decision,
@@ -16,9 +18,12 @@ from nomos.models.agent import (
     Route,
     Step,
     StepIdentifier,
+    StepOverrides,
     Summary,
 )
-from nomos.models.tool import ArgDef, MCPServer, Tool, ToolDef, ToolWrapper
+from nomos.models.tool import Tool, ToolWrapper
+from nomos.tools.mcp import MCPServer
+from nomos.tools.models import ArgDef, ToolDef
 
 
 def test_agent_initialization(basic_agent):
@@ -53,6 +58,48 @@ def test_pkg_tool_registration(basic_agent):
     assert session.tools["combinations"].name == "combinations"
 
 
+def test_api_tool_registration(basic_agent):
+    """Test that API tools are properly registered."""
+
+    # Create an API tool wrapper
+    api_wrapper = ToolWrapper(
+        tool_type="api",
+        name="test_api",
+        tool_identifier="GET/https://api.example.com/users",
+    )
+
+    # Create custom steps that reference the API tool
+    steps = [
+        Step(
+            step_id="start",
+            description="Start step with API tool",
+            routes=[Route(target="end", condition="Task completed")],
+            available_tools=["test_api"],
+        ),
+        Step(step_id="end", description="End step", routes=[], available_tools=[]),
+    ]
+
+    # Create a new agent with the API tool
+    config = AgentConfig(
+        name="api_test_agent",
+        persona="API test persona",
+        steps=steps,
+        start_step_id="start",
+        tools=ToolsConfig(tool_defs={}),
+    )
+
+    agent = Agent.from_config(
+        config=config,
+        llm=basic_agent.llm,
+        tools=[api_wrapper],
+    )
+
+    session = agent.create_session()
+    assert len(session.tools) == 1
+    assert "test_api" in session.tools
+    assert session.tools["test_api"].name == "test_api"
+
+
 def test_basic_conversation_flow(basic_agent, test_tool_0, test_tool_1, tool_defs):
     """Test a basic conversation flow with the agent."""
 
@@ -196,6 +243,89 @@ def test_pkg_tool_usage(basic_agent, test_tool_0, test_tool_1, tool_defs):
     assert any(msg.type == "tool" for msg in messages)
 
 
+@patch("requests.request")
+def test_api_tool_usage(mock_request, basic_agent, test_tool_0, test_tool_1, tool_defs):
+    """Test that the agent can properly use API tools."""
+    # Mock the HTTP response
+    mock_response = Mock()
+    mock_response.text = '{"users": [{"id": 1, "name": "John"}]}'
+    mock_request.return_value = mock_response
+
+    # Create an API tool wrapper
+    api_wrapper = ToolWrapper(
+        tool_type="api",
+        name="get_users",
+        tool_identifier="GET/https://api.example.com/users",
+    )
+
+    # Create custom steps that reference the API tool
+    steps = [
+        Step(
+            step_id="start",
+            description="Start step with API tool",
+            routes=[Route(target="end", condition="Task completed")],
+            available_tools=["get_users"],
+        ),
+        Step(step_id="end", description="End step", routes=[], available_tools=[]),
+    ]
+
+    # Create a new agent with the API tool
+    config = AgentConfig(
+        name="api_test_agent",
+        persona="API test persona",
+        steps=steps,
+        start_step_id="start",
+        tools=ToolsConfig(tool_defs={}),
+    )
+
+    agent = Agent.from_config(
+        config=config,
+        llm=basic_agent.llm,
+        tools=[api_wrapper],
+    )
+
+    session = agent.create_session()
+
+    # Create response models with API tool
+    tool_model = agent.llm._create_decision_model(
+        current_step=session.current_step,
+        current_step_tools=tuple(session.tools.values()),
+    )
+
+    # Set up mock responses
+    tool_response = tool_model(
+        reasoning=["Need to get users from API"],
+        action=Action.TOOL_CALL.value,
+        tool_call={
+            "tool_name": "get_users",
+            "tool_kwargs": {},
+        },
+    )
+
+    session.llm.set_response(tool_response)
+
+    # Tool usage
+    result = session.next("Get users from API", return_tool=True)
+
+    # Verify the tool was called
+    assert result.decision.action == Action.TOOL_CALL
+    assert result.decision.tool_call.tool_name == "get_users"
+    assert result.tool_output == '{"users": [{"id": 1, "name": "John"}]}'
+
+    # Verify HTTP request was made
+    mock_request.assert_called_once_with(
+        method="GET",
+        url="https://api.example.com/users",
+        json=None,
+        headers={},
+        params={},
+    )
+
+    # Verify tool message in history
+    messages = [msg for msg in session.memory.context if isinstance(msg, Event)]
+    assert any(msg.type == "tool" for msg in messages)
+
+
 def test_invalid_tool_args(basic_agent, test_tool_0, test_tool_1, tool_defs):
     """Test handling of invalid tool arguments."""
 
@@ -393,7 +523,7 @@ class TestAgentValidation:
             )
         ]
 
-        with pytest.raises(ValueError, match="Tool nonexistent_tool not found"):
+        with pytest.raises(ValueError):
             Agent(
                 llm=mock_llm,
                 name="test_agent",
@@ -507,7 +637,7 @@ class TestFromConfigErrors:
             start_step_id="start",
         )
 
-        with pytest.raises(ValueError, match="No LLM provided"):
+        with pytest.raises(AssertionError):
             Agent.from_config(config=config)
 
 
@@ -987,10 +1117,7 @@ class TestAgentValidationExtended:
         )
 
         # Should only have one tool with that name
-        tool_names = [
-            tool.name if hasattr(tool, "name") else getattr(tool, "__name__", None)
-            for tool in agent.tools
-        ]
+        tool_names = list(agent.tools.keys())
         assert tool_names.count("same_name") == 1
 
 
@@ -1300,3 +1427,141 @@ class TestDeferredTools:
 
         # make sure that tool_ids are still the same
         assert current_step.tool_ids == []
+
+
+class TestSessionLLM:
+    """Test session LLM functionality."""
+
+    def test_passed_llm(self, mock_llm):
+        """Test that session LLM is passed correctly."""
+        step = Step(
+            step_id="step1",
+            name="test_step",
+            description="init step",
+        )
+        config = AgentConfig(
+            name="agent",
+            persona="You are a helpful assistant",
+            steps=[step],
+            start_step_id="step1",
+            max_examples=2,
+        )
+        agent = Agent.from_config(config=config, llm=mock_llm)
+
+        session = agent.create_session()
+        llm = session.llm
+        assert llm == mock_llm
+
+    def test_no_llm(self):
+        """Test that session raises error if no LLM is provided."""
+        step = Step(
+            step_id="step1",
+            name="test_step",
+            description="init step",
+        )
+        config = AgentConfig(
+            name="agent",
+            persona="You are a helpful assistant",
+            steps=[step],
+            start_step_id="step1",
+            max_examples=2,
+        )
+
+        with pytest.raises(AssertionError):
+            Agent.from_config(config=config)
+
+    def test_config_llm(self, mock_llm):
+        """Test that session LLM is set from config."""
+        llm_config = LLMConfig(
+            provider="openai",
+            model="gpt-3.5-turbo",
+            kwargs={"api_key": "test_key"},
+        )
+
+        step = Step(
+            step_id="step1",
+            name="test_step",
+            description="init step",
+        )
+        config = AgentConfig(
+            name="agent",
+            persona="You are a helpful assistant",
+            steps=[step],
+            start_step_id="step1",
+            llm=llm_config,
+            max_examples=2,
+        )
+        agent = Agent.from_config(config=config)
+
+        session = agent.create_session()
+        llm = session.llm
+        assert isinstance(llm, LLMBase)
+        assert llm.__provider__ == "openai"
+
+
+class TestStepOverrides:
+    """Test StepOverrides functionality."""
+
+    def test_step_persona(self, mock_llm):
+        """Test Step persona overrides session persona."""
+        default_persona = "You are a helpful assistant"
+        custom_persona = "You are an AI Engineer"
+        step_overrides = StepOverrides(persona=custom_persona)
+        step = Step(
+            step_id="step1", name="test_step", description="init step", overrides=step_overrides
+        )
+        config = AgentConfig(
+            name="agent",
+            persona=default_persona,
+            steps=[step],
+            start_step_id="step1",
+            max_examples=2,
+        )
+        agent = Agent.from_config(config=config, llm=mock_llm)
+
+        session = agent.create_session()
+        session.persona = default_persona
+        decision_model = agent.llm._create_decision_model(
+            current_step=session.current_step,
+            current_step_tools=tuple(session._get_current_step_tools()),
+        )
+        response = decision_model(reasoning=["r"], action=Action.RESPOND.value, response="ok")
+        agent.llm.set_response(response)
+
+        session.next("hi")
+        assert custom_persona in session.llm.messages_received[0].content
+        assert default_persona not in session.llm.messages_received[0].content
+
+    def test_get_step_llm_id(self):
+        """Test getting the step LLM ID."""
+        step = Step(
+            step_id="step1",
+            name="test_step",
+            description="init step",
+            overrides=StepOverrides(llm="other"),
+        )
+        config = AgentConfig(
+            name="agent",
+            steps=[step],
+            start_step_id="step1",
+            max_examples=2,
+        )
+        agent = Agent.from_config(
+            config=config,
+            llm={
+                "global": LLMConfig(
+                    provider="openai",
+                    model="gpt-3.5-turbo",
+                    kwargs={"api_key": "test_key"},
+                ),
+                "other": LLMConfig(
+                    provider="openai",
+                    model="gpt-4",
+                    kwargs={"api_key": "test_key_2"},
+                ),
+            },
+        )
+
+        session = agent.create_session()
+        llm = session.llm
+        assert llm.model == "gpt-4"
